=== modified file 'NEWS'
--- NEWS	2006-10-13 07:22:23 +0000
+++ NEWS	2006-10-15 16:22:42 +0000
@@ -34,6 +34,8 @@
       ``--no-plugins`` parameter to ensure test reproducability, and avoid
       problems with system-wide installed plugins. (John Arbash Meinel)
 
+    * Newly-initialized trees have unique root ids.  (Aaron Bentley)
+
   BUG FIXES:
 
     * Don't require ``Content-Type`` in range responses. Assume they are a

=== modified file 'bzrlib/builtins.py'
--- bzrlib/builtins.py	2006-10-13 06:41:04 +0000
+++ bzrlib/builtins.py	2006-10-15 16:22:42 +0000
@@ -1291,6 +1291,8 @@
         for file_id in inv:
             if file_id in basis_inv:
                 continue
+            if inv.is_root(file_id) and len(basis_inv) == 0:
+                continue
             path = inv.id2path(file_id)
             if not os.access(osutils.abspath(path), os.F_OK):
                 continue
@@ -1503,7 +1505,7 @@
             tree = tree.branch.repository.revision_tree(
                 revision[0].in_history(tree.branch).rev_id)
 
-        for fp, fc, kind, fid, entry in tree.list_files():
+        for fp, fc, kind, fid, entry in tree.list_files(include_root=False):
             if fp.startswith(relpath):
                 fp = fp[len(relpath):]
                 if non_recursive and '/' in fp:

=== modified file 'bzrlib/bundle/bundle_data.py'
--- bzrlib/bundle/bundle_data.py	2006-09-26 13:28:39 +0000
+++ bzrlib/bundle/bundle_data.py	2006-10-12 03:19:14 +0000
@@ -648,22 +648,16 @@
 
         assert self.base_tree is not None
         base_inv = self.base_tree.inventory
-        root_id = base_inv.root.file_id
-        try:
-            # New inventories have a unique root_id
-            inv = Inventory(root_id, self.revision_id)
-        except TypeError:
-            inv = Inventory(revision_id=self.revision_id)
-        inv.root.revision = self.get_last_changed(root_id)
+        inv = Inventory(None, self.revision_id)
 
         def add_entry(file_id):
             path = self.id2path(file_id)
             if path is None:
                 return
-            parent_path = dirname(path)
-            if parent_path == u'':
-                parent_id = root_id
+            if path == '':
+                parent_id = None
             else:
+                parent_path = dirname(path)
                 parent_id = self.path2id(parent_path)
 
             kind = self.get_kind(file_id)
@@ -690,8 +684,6 @@
 
         sorted_entries = self.sorted_path_id()
         for path, file_id in sorted_entries:
-            if file_id == inv.root.file_id:
-                continue
             add_entry(file_id)
 
         return inv

=== modified file 'bzrlib/bzrdir.py'
--- bzrlib/bzrdir.py	2006-10-11 00:23:23 +0000
+++ bzrlib/bzrdir.py	2006-10-12 03:19:14 +0000
@@ -685,7 +685,12 @@
         # TODO: jam 20060426 we probably need a test in here in the
         #       case that the newly sprouted branch is a remote one
         if result_repo is None or result_repo.make_working_trees():
-            result.create_workingtree()
+            wt = result.create_workingtree()
+            if wt.inventory.root is None:
+                try:
+                    wt.set_root_id(self.open_workingtree.get_root_id())
+                except errors.NoWorkingTree:
+                    pass
         return result
 
 
@@ -1701,7 +1706,8 @@
                                                   entry_vf=w)
         for old_revision in previous_entries:
                 # if this fails, its a ghost ?
-                assert old_revision in self.converted_revs 
+                assert old_revision in self.converted_revs, \
+                    "Revision {%s} not in converted_revs" % old_revision
         self.snapshot_ie(previous_entries, ie, w, rev_id)
         del ie.text_id
         assert getattr(ie, 'revision', None) is not None

=== modified file 'bzrlib/commit.py'
--- bzrlib/commit.py	2006-09-26 23:15:00 +0000
+++ bzrlib/commit.py	2006-10-15 15:55:32 +0000
@@ -86,7 +86,7 @@
 from bzrlib.testament import Testament
 from bzrlib.trace import mutter, note, warning
 from bzrlib.xml5 import serializer_v5
-from bzrlib.inventory import Inventory, ROOT_ID, InventoryEntry
+from bzrlib.inventory import Inventory, InventoryEntry
 from bzrlib import symbol_versioning
 from bzrlib.symbol_versioning import (deprecated_passed,
         deprecated_function,
@@ -126,6 +126,8 @@
     def snapshot_change(self, change, path):
         if change == 'unchanged':
             return
+        if change == 'added' and path == '':
+            return
         note("%s %s", change, path)
 
     def completed(self, revno, rev_id):
@@ -390,6 +392,10 @@
         # A merge with no effect on files
         if len(self.parents) > 1:
             return
+        # work around the fact that a newly-initted tree does differ from its
+        # basis
+        if len(self.basis_inv) == 0 and len(self.builder.new_inventory) == 1:
+            raise PointlessCommit()
         # Shortcut, if the number of entries changes, then we obviously have
         # a change
         if len(self.builder.new_inventory) != len(self.basis_inv):
@@ -550,6 +556,7 @@
         # in bugs like #46635.  Any reason not to use/enhance Tree.changes_from?
         # ADHB 11-07-2006
         mutter("Selecting files for commit with filter %s", self.specific_files)
+        assert self.work_inv.root is not None
         entries = self.work_inv.iter_entries()
         if not self.builder.record_root_entry:
             symbol_versioning.warn('CommitBuilders should support recording'
@@ -574,7 +581,6 @@
                 else:
                     # this entry is new and not being committed
                     continue
-
             self.builder.record_entry_contents(ie, self.parent_invs, 
                 path, self.work_tree)
             # describe the nature of the change that has occurred relative to

=== modified file 'bzrlib/delta.py'
--- bzrlib/delta.py	2006-09-29 20:27:19 +0000
+++ bzrlib/delta.py	2006-10-12 03:19:14 +0000
@@ -152,14 +152,14 @@
         want_unchanged=want_unchanged,
         specific_files=specific_files,
         extra_trees=extra_trees,
-        require_versioned=require_versioned)
+        require_versioned=require_versioned,
+        include_root=False)
 
 
 def _compare_trees(old_tree, new_tree, want_unchanged, specific_file_ids,
                    include_root):
 
     from osutils import is_inside_any
-    
     old_inv = old_tree.inventory
     new_inv = new_tree.inventory
     delta = TreeDelta()

=== modified file 'bzrlib/errors.py'
--- bzrlib/errors.py	2006-10-11 05:55:32 +0000
+++ bzrlib/errors.py	2006-10-15 16:22:42 +0000
@@ -1020,6 +1020,18 @@
     """Tree transform is malformed %(conflicts)r"""
 
 
+class NoFinalPath(BzrNewError):
+    """No final name for trans_id %(trans_id)r
+    file-id: %(file_id)r"
+    root trans-id: %(root_trans_id)r 
+    """
+
+    def __init__(self, trans_id, transform):
+        self.trans_id = trans_id
+        self.file_id = transform.final_file_id(trans_id)
+        self.root_trans_id = transform.root
+
+
 class BzrBadParameter(BzrNewError):
     """A bad parameter : %(param)s is not usable.
     

=== modified file 'bzrlib/info.py'
--- bzrlib/info.py	2006-09-08 18:52:17 +0000
+++ bzrlib/info.py	2006-09-10 19:04:48 +0000
@@ -210,9 +210,10 @@
     print '  %8d ignored' % ignore_cnt
 
     dir_cnt = 0
-    entries = work_inv.iter_entries()
-    entries.next()
-    dir_cnt = sum(1 for path, ie in entries if ie.kind == 'directory')
+    for file_id in work_inv:
+        if (work_inv.get_file_kind(file_id) == 'directory' and 
+            not work_inv.is_root(file_id)):
+            dir_cnt += 1
     print '  %8d versioned %s' \
           % (dir_cnt,
              plural(dir_cnt, 'subdirectory', 'subdirectories'))

=== modified file 'bzrlib/inventory.py'
--- bzrlib/inventory.py	2006-10-06 05:53:44 +0000
+++ bzrlib/inventory.py	2006-10-12 01:50:46 +0000
@@ -855,6 +855,9 @@
     ['', u'hello.c']
     >>> inv = Inventory('TREE_ROOT-12345678-12345678')
     >>> inv.add(InventoryFile('123-123', 'hello.c', ROOT_ID))
+    Traceback (most recent call last):
+    BzrError: parent_id {TREE_ROOT} not in inventory
+    >>> inv.add(InventoryFile('123-123', 'hello.c', 'TREE_ROOT-12345678-12345678'))
     InventoryFile('123-123', 'hello.c', parent_id='TREE_ROOT-12345678-12345678', sha1=None, len=None)
     """
     def __init__(self, root_id=ROOT_ID, revision_id=None):
@@ -867,17 +870,11 @@
         The inventory is created with a default root directory, with
         an id of None.
         """
-        # We are letting Branch.create() create a unique inventory
-        # root id. Rather than generating a random one here.
-        #if root_id is None:
-        #    root_id = bzrlib.branch.gen_file_id('TREE_ROOT')
         if root_id is not None:
             self._set_root(InventoryDirectory(root_id, '', None))
         else:
             self.root = None
             self._byid = {}
-        # FIXME: this isn't ever used, changing it to self.revision may break
-        # things. TODO make everything use self.revision_id
         self.revision_id = revision_id
 
     def _set_root(self, ie):
@@ -904,7 +901,8 @@
     def iter_entries(self, from_dir=None):
         """Return (path, entry) pairs, in order by name."""
         if from_dir is None:
-            assert self.root
+            if self.root is None:
+                return
             from_dir = self.root
             yield '', self.root
         elif isinstance(from_dir, basestring):
@@ -957,7 +955,8 @@
         # TODO? Perhaps this should return the from_dir so that the root is
         # yielded? or maybe an option?
         if from_dir is None:
-            assert self.root
+            if self.root is None:
+                return
             from_dir = self.root
             yield '', self.root
         elif isinstance(from_dir, basestring):
@@ -1063,10 +1062,6 @@
             assert self.root is None and len(self._byid) == 0
             self._set_root(entry)
             return entry
-        if entry.parent_id == ROOT_ID:
-            assert self.root is not None, self
-            entry.parent_id = self.root.file_id
-
         try:
             parent = self._byid[entry.parent_id]
         except KeyError:
@@ -1205,6 +1200,8 @@
         # mutter("lookup path %r" % name)
 
         parent = self.root
+        if parent is None:
+            return None
         for f in name:
             try:
                 cie = parent.children[f]
@@ -1270,6 +1267,9 @@
         file_ie.name = new_name
         file_ie.parent_id = new_parent_id
 
+    def is_root(self, file_id):
+        return self.root is not None and file_id == self.root.file_id
+
 
 def make_entry(kind, name, parent_id, file_id=None):
     """Create an inventory entry.

=== modified file 'bzrlib/memorytree.py'
--- bzrlib/memorytree.py	2006-09-15 02:03:15 +0000
+++ bzrlib/memorytree.py	2006-09-22 04:52:17 +0000
@@ -167,8 +167,9 @@
         self._file_transport = MemoryTransport()
         # TODO copy the revision trees content, or do it lazy, or something.
         inventory_entries = self._inventory.iter_entries()
-        inventory_entries.next()
         for path, entry in inventory_entries:
+            if path == '':
+                continue
             if entry.kind == 'directory':
                 self._file_transport.mkdir(path)
             elif entry.kind == 'file':

=== modified file 'bzrlib/merge.py'
--- bzrlib/merge.py	2006-10-06 05:53:44 +0000
+++ bzrlib/merge.py	2006-10-12 01:50:46 +0000
@@ -45,7 +45,8 @@
 from bzrlib.textfile import check_text_lines
 from bzrlib.trace import mutter, warning, note
 from bzrlib.transform import (TreeTransform, resolve_conflicts, cook_conflicts,
-                              FinalPaths, create_by_entry, unique_add)
+                              FinalPaths, create_by_entry, unique_add,
+                              ROOT_PARENT)
 from bzrlib.versionedfile import WeaveMerge
 from bzrlib import ui
 
@@ -388,7 +389,7 @@
                     self.merge_executable(file_id, file_status)
             finally:
                 child_pb.finished()
-                
+            self.fix_root()
             self.pp.next_phase()
             child_pb = ui.ui_factory.nested_progress_bar()
             try:
@@ -410,6 +411,33 @@
             working_tree.unlock()
             self.pb.clear()
 
+    def fix_root(self):
+        try:
+            self.tt.final_kind(self.tt.root)
+        except NoSuchFile:
+            self.tt.cancel_deletion(self.tt.root)
+        if self.tt.final_file_id(self.tt.root) is None:
+            self.tt.version_file(self.tt.tree_file_id(self.tt.root), 
+                                 self.tt.root)
+        if self.other_tree.inventory.root is None:
+            return
+        other_root_file_id = self.other_tree.inventory.root.file_id
+        other_root = self.tt.trans_id_file_id(other_root_file_id)
+        if other_root == self.tt.root:
+            return
+        try:
+            self.tt.final_kind(other_root)
+        except NoSuchFile:
+            return
+        self.reparent_children(self.other_tree.inventory.root, self.tt.root)
+        self.tt.cancel_creation(other_root)
+        self.tt.cancel_versioning(other_root)
+
+    def reparent_children(self, ie, target):
+        for thing, child in ie.children.iteritems():
+            trans_id = self.tt.trans_id_file_id(child.file_id)
+            self.tt.adjust_path(self.tt.final_name(trans_id), target, trans_id)
+
     def write_modified(self, results):
         modified_hashes = {}
         for path in results.modified_paths:
@@ -520,9 +548,10 @@
                         "conflict": other_entry}
         trans_id = self.tt.trans_id_file_id(file_id)
         parent_id = winner_entry[parent_id_winner].parent_id
-        parent_trans_id = self.tt.trans_id_file_id(parent_id)
-        self.tt.adjust_path(winner_entry[name_winner].name, parent_trans_id,
-                            trans_id)
+        if parent_id is not None:
+            parent_trans_id = self.tt.trans_id_file_id(parent_id)
+            self.tt.adjust_path(winner_entry[name_winner].name, 
+                                parent_trans_id, trans_id)
 
     def merge_contents(self, file_id):
         """Performa a merge on file_id contents."""

=== modified file 'bzrlib/mutabletree.py'
--- bzrlib/mutabletree.py	2006-09-15 02:03:15 +0000
+++ bzrlib/mutabletree.py	2006-09-22 04:52:17 +0000
@@ -100,8 +100,6 @@
             if self.is_control_filename(f):
                 raise errors.ForbiddenControlFileError(filename=f)
             fp = splitpath(f)
-            if len(fp) == 0:
-                raise BzrError("cannot add top-level %r" % f)
         # fill out file kinds for all files [not needed when we stop 
         # caring about the instantaneous file kind within a uncommmitted tree
         #

=== modified file 'bzrlib/repository.py'
--- bzrlib/repository.py	2006-10-06 05:53:44 +0000
+++ bzrlib/repository.py	2006-10-12 01:50:46 +0000
@@ -605,7 +605,8 @@
         # TODO: refactor this to use an existing revision object
         # so we don't need to read it in twice.
         if revision_id is None or revision_id == _mod_revision.NULL_REVISION:
-            return RevisionTree(self, Inventory(), _mod_revision.NULL_REVISION)
+            return RevisionTree(self, Inventory(root_id=None), 
+                                _mod_revision.NULL_REVISION)
         else:
             inv = self.get_revision_inventory(revision_id)
             return RevisionTree(self, inv, revision_id)

=== modified file 'bzrlib/revisiontree.py'
--- bzrlib/revisiontree.py	2006-09-21 23:23:03 +0000
+++ bzrlib/revisiontree.py	2006-09-28 01:53:23 +0000
@@ -40,7 +40,6 @@
         self._repository = branch
         self._weave_store = branch.weave_store
         self._inventory = inv
-        assert inv.root is not None
         self._revision_id = revision_id
 
     def get_parent_ids(self):
@@ -100,7 +99,8 @@
     def list_files(self, include_root=False):
         # The only files returned by this are those from the version
         entries = self.inventory.iter_entries()
-        if not include_root:
+        # skip the root for compatability with the current apis.
+        if self.inventory.root is not None and not include_root:
             # skip the root for compatability with the current apis.
             entries.next()
         for path, entry in entries:

=== modified file 'bzrlib/tests/bzrdir_implementations/test_bzrdir.py'
--- bzrlib/tests/bzrdir_implementations/test_bzrdir.py	2006-09-29 16:34:45 +0000
+++ bzrlib/tests/bzrdir_implementations/test_bzrdir.py	2006-10-02 13:05:18 +0000
@@ -17,6 +17,7 @@
 """Tests for bzrdir implementations - tests a bzrdir format."""
 
 from cStringIO import StringIO
+import errno
 import os
 from stat import S_ISDIR
 import sys
@@ -536,11 +537,28 @@
         repo = dir.create_repository()
         repo.fetch(tree.branch.repository)
         self.assertTrue(repo.has_revision('1'))
+        try:
+            self.assertIs(dir.open_branch().last_revision(), None)
+        except errors.NotBranchError:
+            pass
         target = self.sproutOrSkip(dir, self.get_url('target'))
         self.assertNotEqual(dir.transport.base, target.transport.base)
+        # testing inventory isn't reasonable for repositories
         self.assertDirectoriesEqual(dir.root_transport, target.root_transport,
                                     ['./.bzr/repository/inventory.knit',
+                                     './.bzr/inventory'
                                      ])
+        try:
+            # If we happen to have a tree, we'll guarantee everything
+            # except for the tree root is the same.
+            inventory_f = file(dir.transport.base+'inventory', 'rb')
+            self.assertContainsRe(inventory_f.read(), 
+                                  '<inventory file_id="TREE_ROOT[^"]*"'
+                                  ' format="5">\n</inventory>\n')
+            inventory_f.close()
+        except IOError, e:
+            if e.errno != errno.ENOENT:
+                raise
 
     def test_sprout_bzrdir_with_repository_to_shared(self):
         tree = self.make_branch_and_tree('commit_tree')

=== modified file 'bzrlib/tests/repository_implementations/test_fileid_involved.py'
--- bzrlib/tests/repository_implementations/test_fileid_involved.py	2006-09-09 18:52:57 +0000
+++ bzrlib/tests/repository_implementations/test_fileid_involved.py	2006-09-10 20:12:00 +0000
@@ -176,9 +176,10 @@
     def fileids_altered_by_revision_ids(self, revision_ids):
         """This is a wrapper to strip TREE_ROOT if it occurs"""
         repo = self.branch.repository
+        root_id = self.branch.basis_tree().inventory.root.file_id
         result = repo.fileids_altered_by_revision_ids(revision_ids)
-        if 'TREE_ROOT' in result:
-            del result['TREE_ROOT']
+        if root_id in result:
+            del result[root_id]
         return result
 
     def test_fileids_altered_by_revision_ids(self):

=== modified file 'bzrlib/tests/repository_implementations/test_repository.py'
--- bzrlib/tests/repository_implementations/test_repository.py	2006-09-29 11:15:51 +0000
+++ bzrlib/tests/repository_implementations/test_repository.py	2006-10-02 13:05:18 +0000
@@ -30,7 +30,7 @@
                            UninitializableFormat,
                            NotBranchError,
                            )
-from bzrlib.inventory import Inventory
+from bzrlib.inventory import Inventory, InventoryDirectory
 from bzrlib.revision import NULL_REVISION
 from bzrlib.tests import TestCase, TestCaseWithTransport, TestSkipped
 from bzrlib.tests.bzrdir_implementations.test_bzrdir import TestCaseWithBzrDir
@@ -153,13 +153,18 @@
 
     def test_revision_tree(self):
         wt = self.make_branch_and_tree('.')
+        wt.set_root_id('fixed-root')
         wt.commit('lala!', rev_id='revision-1', allow_pointless=True)
         tree = wt.branch.repository.revision_tree('revision-1')
-        self.assertEqual(list(tree.list_files()), [])
+        self.assertEqual('revision-1', tree.inventory.root.revision) 
+        expected = InventoryDirectory('fixed-root', '', None)
+        expected.revision = 'revision-1'
+        self.assertEqual([('', 'V', 'directory', 'fixed-root', expected)],
+                         list(tree.list_files(include_root=True)))
         tree = wt.branch.repository.revision_tree(None)
-        self.assertEqual([], list(tree.list_files()))
+        self.assertEqual([], list(tree.list_files(include_root=True)))
         tree = wt.branch.repository.revision_tree(NULL_REVISION)
-        self.assertEqual([], list(tree.list_files()))
+        self.assertEqual([], list(tree.list_files(include_root=True)))
 
     def test_fetch(self):
         # smoke test fetch to ensure that the convenience function works.

=== modified file 'bzrlib/tests/test_ancestry.py'
--- bzrlib/tests/test_ancestry.py	2006-10-04 04:50:23 +0000
+++ bzrlib/tests/test_ancestry.py	2006-10-15 15:55:32 +0000
@@ -35,8 +35,11 @@
         """Test ancestry file when just committing."""
         tree = self.make_branch_and_memory_tree('.')
         branch = tree.branch
+        tree.lock_write()
+        tree.add('')
         rev_id_one = tree.commit('one')
         rev_id_two = tree.commit('two', allow_pointless=True)
+        tree.unlock()
 
         self.assertAncestryEqual([None, rev_id_one, rev_id_two],
             rev_id_two, branch)
@@ -44,9 +47,12 @@
 
     def test_none_is_always_an_ancestor(self):
         tree = self.make_branch_and_memory_tree('.')
+        tree.lock_write()
+        tree.add('')
         # note this is tested before any commits are done.
         self.assertTrue(is_ancestor(None, None, tree.branch))
         rev_id = tree.commit('one')
+        tree.unlock()
         self.assertTrue(is_ancestor(None, None, tree.branch))
         self.assertTrue(is_ancestor(rev_id, None, tree.branch))
         self.assertFalse(is_ancestor(None, rev_id, tree.branch))

=== modified file 'bzrlib/tests/test_commit_merge.py'
--- bzrlib/tests/test_commit_merge.py	2006-09-04 08:00:00 +0000
+++ bzrlib/tests/test_commit_merge.py	2006-09-10 19:04:48 +0000
@@ -63,7 +63,9 @@
         wtx = self.make_branch_and_tree('x')
         base_rev = wtx.commit('common parent')
         bx = wtx.branch
+        wtx.commit('establish root id')
         wty = wtx.bzrdir.sprout('y').open_workingtree()
+        self.assertEqual(wtx.get_root_id(), wty.get_root_id())
         by = wty.branch
 
         self.build_tree(['x/ecks', 'y/why'])

=== modified file 'bzrlib/tests/test_inv.py'
--- bzrlib/tests/test_inv.py	2006-08-23 10:30:07 +0000
+++ bzrlib/tests/test_inv.py	2006-09-03 20:41:45 +0000
@@ -544,3 +544,19 @@
         os.unlink('b1/a')
         wt.revert([])
         self.assertEqual(len(wt.inventory), 1)
+
+
+class TestIsRoot(TestCase):
+    """Ensure our root-checking code is accurate."""
+
+    def test_is_root(self):
+        inv = Inventory('TREE_ROOT')
+        self.assertTrue(inv.is_root('TREE_ROOT'))
+        self.assertFalse(inv.is_root('booga'))
+        inv.root.file_id = 'booga'
+        self.assertFalse(inv.is_root('TREE_ROOT'))
+        self.assertTrue(inv.is_root('booga'))
+        # works properly even if no root is set
+        inv.root = None
+        self.assertFalse(inv.is_root('TREE_ROOT'))
+        self.assertFalse(inv.is_root('booga'))

=== modified file 'bzrlib/tests/test_memorytree.py'
--- bzrlib/tests/test_memorytree.py	2006-09-18 05:38:58 +0000
+++ bzrlib/tests/test_memorytree.py	2006-10-15 15:55:32 +0000
@@ -86,7 +86,8 @@
         branch = self.make_branch('branch')
         tree = MemoryTree.create_on_branch(branch)
         tree.lock_write()
-        tree.add(['afile', 'adir'], None, ['file', 'directory'])
+        tree.add(['', 'afile', 'adir'], None, 
+                 ['directory', 'file', 'directory'])
         self.assertEqual('afile', tree.id2path(tree.path2id('afile')))
         self.assertEqual('adir', tree.id2path(tree.path2id('adir')))
         self.assertFalse(tree.has_filename('afile'))
@@ -97,7 +98,8 @@
         branch = self.make_branch('branch')
         tree = MemoryTree.create_on_branch(branch)
         tree.lock_write()
-        tree.add(['foo'], ids=['foo-id'], kinds=['file'])
+        tree.add(['', 'foo'], ids=['root-id', 'foo-id'], 
+                  kinds=['directory', 'file'])
         tree.put_file_bytes_non_atomic('foo-id', 'barshoom')
         self.assertEqual('barshoom', tree.get_file('foo-id').read())
         tree.unlock()
@@ -106,7 +108,8 @@
         branch = self.make_branch('branch')
         tree = MemoryTree.create_on_branch(branch)
         tree.lock_write()
-        tree.add(['foo'], ids=['foo-id'], kinds=['file'])
+        tree.add(['', 'foo'], ids=['root-id', 'foo-id'], 
+                 kinds=['directory', 'file'])
         tree.put_file_bytes_non_atomic('foo-id', 'first-content')
         tree.put_file_bytes_non_atomic('foo-id', 'barshoom')
         self.assertEqual('barshoom', tree.get_file('foo-id').read())
@@ -121,7 +124,8 @@
         branch = self.make_branch('branch')
         tree = MemoryTree.create_on_branch(branch)
         tree.lock_write()
-        tree.add(['foo'], ids=['foo-id'], kinds=['file'])
+        tree.add(['', 'foo'], ids=['root-id', 'foo-id'], 
+                 kinds=['directory', 'file'])
         tree.put_file_bytes_non_atomic('foo-id', 'barshoom')
         revision_id = tree.commit('message baby')
         # the parents list for the tree should have changed.
@@ -136,7 +140,8 @@
         branch = self.make_branch('branch')
         tree = MemoryTree.create_on_branch(branch)
         tree.lock_write()
-        tree.add(['foo'], ids=['foo-id'], kinds=['file'])
+        tree.add(['', 'foo'], ids=['root-id', 'foo-id'], 
+                 kinds=['directory', 'file'])
         tree.unversion(['foo-id'])
         self.assertFalse(tree.has_id('foo-id'))
         tree.unlock()
@@ -144,5 +149,8 @@
     def test_last_revision(self):
         """There should be a last revision method we can call."""
         tree = self.make_branch_and_memory_tree('branch')
+        tree.lock_write()
+        tree.add('')
         rev_id = tree.commit('first post')
+        tree.unlock()
         self.assertEqual(rev_id, tree.last_revision())

=== modified file 'bzrlib/tests/test_merge_core.py'
--- bzrlib/tests/test_merge_core.py	2006-10-06 05:53:44 +0000
+++ bzrlib/tests/test_merge_core.py	2006-10-12 01:50:46 +0000
@@ -18,18 +18,20 @@
                             )
 from bzrlib.transform import TreeTransform
 from bzrlib.tests import TestCaseWithTransport, TestCase, TestSkipped
-from bzrlib.workingtree import WorkingTree
+from bzrlib.workingtree import WorkingTree, gen_root_id
 
 
 class MergeBuilder(object):
     def __init__(self, dir=None):
         self.dir = osutils.mkdtemp(prefix="merge-test", dir=dir)
+        self.tree_root = gen_root_id()
         def wt(name):
            path = pathjoin(self.dir, name)
            os.mkdir(path)
            wt = bzrlib.bzrdir.BzrDir.create_standalone_workingtree(path)
            # the tests perform pulls, so need a branch that is writeable.
            wt.lock_write()
+           wt.set_root_id(self.tree_root)
            tt = TreeTransform(wt)
            return wt, tt
         self.base, self.base_tt = wt('base')
@@ -182,16 +184,16 @@
     def test_change_name(self):
         """Test renames"""
         builder = MergeBuilder(getcwd())
-        builder.add_file("1", "TREE_ROOT", "name1", "hello1", True)
+        builder.add_file("1", builder.tree_root, "name1", "hello1", True)
         builder.change_name("1", other="name2")
-        builder.add_file("2", "TREE_ROOT", "name3", "hello2", True)
+        builder.add_file("2", builder.tree_root, "name3", "hello2", True)
         builder.change_name("2", base="name4")
-        builder.add_file("3", "TREE_ROOT", "name5", "hello3", True)
+        builder.add_file("3", builder.tree_root, "name5", "hello3", True)
         builder.change_name("3", this="name6")
         builder.merge()
         builder.cleanup()
         builder = MergeBuilder(getcwd())
-        builder.add_file("1", "TREE_ROOT", "name1", "hello1", False)
+        builder.add_file("1", builder.tree_root, "name1", "hello1", False)
         builder.change_name("1", other="name2", this="name3")
         conflicts = builder.merge()
         self.assertEqual(conflicts, [PathConflict('name3', 'name2', '1')])
@@ -199,9 +201,9 @@
 
     def test_merge_one(self):
         builder = MergeBuilder(getcwd())
-        builder.add_file("1", "TREE_ROOT", "name1", "hello1", True)
+        builder.add_file("1", builder.tree_root, "name1", "hello1", True)
         builder.change_contents("1", other="text4")
-        builder.add_file("2", "TREE_ROOT", "name2", "hello1", True)
+        builder.add_file("2", builder.tree_root, "name2", "hello1", True)
         builder.change_contents("2", other="text4")
         builder.merge(interesting_ids=["1"])
         self.assertEqual(builder.this.get_file("1").read(), "text4" )
@@ -211,8 +213,8 @@
     def test_file_moves(self):
         """Test moves"""
         builder = MergeBuilder(getcwd())
-        builder.add_dir("1", "TREE_ROOT", "dir1")
-        builder.add_dir("2", "TREE_ROOT", "dir2")
+        builder.add_dir("1", builder.tree_root, "dir1")
+        builder.add_dir("2", builder.tree_root, "dir2")
         builder.add_file("3", "1", "file1", "hello1", True)
         builder.add_file("4", "1", "file2", "hello2", True)
         builder.add_file("5", "1", "file3", "hello3", True)
@@ -223,9 +225,9 @@
         builder.cleanup()
 
         builder = MergeBuilder(getcwd())
-        builder.add_dir("1", "TREE_ROOT", "dir1")
-        builder.add_dir("2", "TREE_ROOT", "dir2")
-        builder.add_dir("3", "TREE_ROOT", "dir3")
+        builder.add_dir("1", builder.tree_root, "dir1")
+        builder.add_dir("2", builder.tree_root, "dir2")
+        builder.add_dir("3", builder.tree_root, "dir3")
         builder.add_file("4", "1", "file1", "hello1", False)
         builder.change_parent("4", other="2", this="3")
         conflicts = builder.merge()
@@ -252,7 +254,7 @@
     def test_reprocess_weave(self):
         # Reprocess works on weaves, and behaves as expected
         builder = MergeBuilder(getcwd())
-        builder.add_file('a', 'TREE_ROOT', 'blah', 'a', False)
+        builder.add_file('a', builder.tree_root, 'blah', 'a', False)
         builder.change_contents('a', this='b\nc\nd\ne\n', other='z\nc\nd\ny\n')
         builder.merge(WeaveMerger, reprocess=True)
         expected = """<<<<<<< TREE
@@ -279,14 +281,15 @@
 
     def contents_test_success(self, merge_factory):
         builder = MergeBuilder(getcwd())
-        builder.add_file("1", "TREE_ROOT", "name1", "text1", True)
+        builder.add_file("1", builder.tree_root, "name1", "text1", True)
         builder.change_contents("1", other="text4")
-        builder.add_file("2", "TREE_ROOT", "name3", "text2", False)
+        builder.add_file("2", builder.tree_root, "name3", "text2", False)
         builder.change_contents("2", base="text5")
-        builder.add_file("3", "TREE_ROOT", "name5", "text3", True)
-        builder.add_file("4", "TREE_ROOT", "name6", "text4", True)
+        builder.add_file("3", builder.tree_root, "name5", "text3", True)
+        builder.add_file("4", builder.tree_root, "name6", "text4", True)
         builder.remove_file("4", base=True)
-        builder.add_file("5", "TREE_ROOT", "name7", "a\nb\nc\nd\ne\nf\n", True)
+        builder.add_file("5", builder.tree_root, "name7", "a\nb\nc\nd\ne\nf\n",
+                         True)
         builder.change_contents("5", other="a\nz\nc\nd\ne\nf\n", 
                                      this="a\nb\nc\nd\ne\nz\n")
         conflicts = builder.merge(merge_factory)
@@ -306,11 +309,11 @@
 
     def contents_test_conflicts(self, merge_factory):
         builder = MergeBuilder(getcwd())
-        builder.add_file("1", "TREE_ROOT", "name1", "text1", True)
+        builder.add_file("1", builder.tree_root, "name1", "text1", True)
         builder.change_contents("1", other="text4", this="text3")
-        builder.add_file("2", "TREE_ROOT", "name2", "text1", True)
+        builder.add_file("2", builder.tree_root, "name2", "text1", True)
         builder.change_contents("2", other="\x00", this="text3")
-        builder.add_file("3", "TREE_ROOT", "name3", "text5", False)
+        builder.add_file("3", builder.tree_root, "name3", "text5", False)
         builder.change_perms("3", this=True)
         builder.change_contents('3', this='moretext')
         builder.remove_file('3', other=True)
@@ -324,7 +327,7 @@
     def test_symlink_conflicts(self):
         if sys.platform != "win32":
             builder = MergeBuilder(getcwd())
-            builder.add_symlink("2", "TREE_ROOT", "name2", "target1")
+            builder.add_symlink("2", builder.tree_root, "name2", "target1")
             builder.change_target("2", other="target4", base="text3")
             conflicts = builder.merge()
             self.assertEqual(conflicts, [ContentsConflict('name2', 
@@ -334,9 +337,9 @@
     def test_symlink_merge(self):
         if sys.platform != "win32":
             builder = MergeBuilder(getcwd())
-            builder.add_symlink("1", "TREE_ROOT", "name1", "target1")
-            builder.add_symlink("2", "TREE_ROOT", "name2", "target1")
-            builder.add_symlink("3", "TREE_ROOT", "name3", "target1")
+            builder.add_symlink("1", builder.tree_root, "name1", "target1")
+            builder.add_symlink("2", builder.tree_root, "name2", "target1")
+            builder.add_symlink("3", builder.tree_root, "name3", "target1")
             builder.change_target("1", this="target2")
             builder.change_target("2", base="target2")
             builder.change_target("3", other="target2")
@@ -348,20 +351,20 @@
 
     def test_no_passive_add(self):
         builder = MergeBuilder(getcwd())
-        builder.add_file("1", "TREE_ROOT", "name1", "text1", True)
+        builder.add_file("1", builder.tree_root, "name1", "text1", True)
         builder.remove_file("1", this=True)
         builder.merge()
         builder.cleanup()
 
     def test_perms_merge(self):
         builder = MergeBuilder(getcwd())
-        builder.add_file("1", "TREE_ROOT", "name1", "text1", True)
+        builder.add_file("1", builder.tree_root, "name1", "text1", True)
         builder.change_perms("1", other=False)
-        builder.add_file("2", "TREE_ROOT", "name2", "text2", True)
+        builder.add_file("2", builder.tree_root, "name2", "text2", True)
         builder.change_perms("2", base=False)
-        builder.add_file("3", "TREE_ROOT", "name3", "text3", True)
+        builder.add_file("3", builder.tree_root, "name3", "text3", True)
         builder.change_perms("3", this=False)
-        builder.add_file('4', 'TREE_ROOT', 'name4', 'text4', False)
+        builder.add_file('4', builder.tree_root, 'name4', 'text4', False)
         builder.change_perms('4', this=True)
         builder.remove_file('4', base=True)
         builder.merge()
@@ -372,19 +375,19 @@
 
     def test_new_suffix(self):
         builder = MergeBuilder(getcwd())
-        builder.add_file("1", "TREE_ROOT", "name1", "text1", True)
+        builder.add_file("1", builder.tree_root, "name1", "text1", True)
         builder.change_contents("1", other="text3")
-        builder.add_file("2", "TREE_ROOT", "name1.new", "text2", True)
+        builder.add_file("2", builder.tree_root, "name1.new", "text2", True)
         builder.merge()
         os.lstat(builder.this.id2abspath("2"))
         builder.cleanup()
 
     def test_spurious_conflict(self):
         builder = MergeBuilder(getcwd())
-        builder.add_file("1", "TREE_ROOT", "name1", "text1", False)
+        builder.add_file("1", builder.tree_root, "name1", "text1", False)
         builder.remove_file("1", other=True)
-        builder.add_file("2", "TREE_ROOT", "name1", "text1", False, this=False, 
-                         base=False)
+        builder.add_file("2", builder.tree_root, "name1", "text1", False, 
+                         this=False, base=False)
         conflicts = builder.merge()
         self.assertEqual(conflicts, []) 
         builder.cleanup()

=== modified file 'bzrlib/tests/test_revisiontree.py'
--- bzrlib/tests/test_revisiontree.py	2006-09-07 23:13:08 +0000
+++ bzrlib/tests/test_revisiontree.py	2006-09-30 00:25:31 +0000
@@ -17,6 +17,9 @@
 
 """Tests for the RevisionTree class."""
 
+from bzrlib import (
+    revision,
+    )
 import bzrlib
 from bzrlib.tests import TestCaseWithTransport
 from bzrlib.tree import RevisionTree
@@ -51,3 +54,8 @@
         # the parents for a revision_tree(None) are []:
         self.assertEqual([],
             self.t.branch.repository.revision_tree(None).get_parent_ids())
+
+    def test_empty_no_root(self):
+        null_tree = self.t.branch.repository.revision_tree(
+            revision.NULL_REVISION)
+        self.assertIs(None, null_tree.inventory.root)

=== modified file 'bzrlib/tests/test_smart_add.py'
--- bzrlib/tests/test_smart_add.py	2006-09-07 21:37:43 +0000
+++ bzrlib/tests/test_smart_add.py	2006-09-10 19:04:48 +0000
@@ -319,14 +319,12 @@
 
         self.build_tree(['new/a', 'new/b', 'new/c',
                          'new/subdir/', 'new/subdir/b', 'new/subdir/d'])
+        new_tree.set_root_id(self.base_tree.get_root_id())
         self.add_helper(self.base_tree, 'dir', new_tree, ['new'])
 
-        # We 'a' and 'b' exist in the root, and they are being added
-        # in a new 'root'. Since ROOT ids are not unique, we will
+        # We know 'a' and 'b' exist in the root, and they are being added
+        # in a new 'root'. Since ROOT ids have been set as the same, we will
         # use those ids
-        # TODO: This will probably change once trees have a unique root id
-        # It is definitely arguable that 'a' should get the id of
-        # 'dir/a' not of 'a'.
         self.assertEqual(self.base_tree.path2id('a'),
                          new_tree.path2id('a'))
         self.assertEqual(self.base_tree.path2id('b'),

=== modified file 'bzrlib/tests/test_testament.py'
--- bzrlib/tests/test_testament.py	2006-09-26 13:28:39 +0000
+++ bzrlib/tests/test_testament.py	2006-09-28 01:57:43 +0000
@@ -32,6 +32,7 @@
     def setUp(self):
         super(TestamentSetup, self).setUp()
         self.wt = self.make_branch_and_tree('.')
+        self.wt.set_root_id('TREE_ROT')
         b = self.b = self.wt.branch
         b.nick = "test branch"
         self.wt.commit(message='initial null commit',
@@ -201,7 +202,7 @@
 message:
   initial null commit
 inventory:
-  directory . TREE_ROOT test@user-1 no
+  directory . TREE_ROT test@user-1 no
 properties:
   branch-nick:
     test branch
@@ -280,7 +281,7 @@
 message:
   add files and directories
 inventory:
-  directory . TREE_ROOT test@user-2 no
+  directory . TREE_ROT test@user-2 no
   file hello hello-id 34dd0ac19a24bf80c4d33b5c8960196e8d8d1f73 test@user-2 yes
   directory src src-id test@user-2 no
   file src/foo.c foo.c-id a2a049c20f908ae31b231d98779eb63c66448f24 test@user-2 no
@@ -374,7 +375,7 @@
 message:
   revision with properties
 inventory:
-  directory . TREE_ROOT test@user-3 no
+  directory . TREE_ROT test@user-3 no
   file hello hello-id 34dd0ac19a24bf80c4d33b5c8960196e8d8d1f73 test@user-2 yes
   directory src src-id test@user-2 no
   file src/foo.c foo.c-id a2a049c20f908ae31b231d98779eb63c66448f24 test@user-2 no
@@ -443,7 +444,7 @@
 message:
   add symlink
 inventory:
-  directory . TREE_ROOT test@user-3 no
+  directory . TREE_ROT test@user-3 no
   file hello hello-id 34dd0ac19a24bf80c4d33b5c8960196e8d8d1f73 test@user-2 yes
   symlink link link-id wibble/linktarget test@user-3 no
   directory src src-id test@user-2 no
@@ -509,7 +510,7 @@
 message:
   non-ascii commit \N{COPYRIGHT SIGN} me
 inventory:
-  directory . TREE_ROOT test@user-3 no
+  directory . TREE_ROT test@user-3 no
   file hello hello-id 34dd0ac19a24bf80c4d33b5c8960196e8d8d1f73 test@user-2 yes
   directory src src-id test@user-2 no
   file src/foo.c foo.c-id a2a049c20f908ae31b231d98779eb63c66448f24 test@user-2 no

=== modified file 'bzrlib/tests/test_transform.py'
--- bzrlib/tests/test_transform.py	2006-09-21 20:21:36 +0000
+++ bzrlib/tests/test_transform.py	2006-09-27 02:28:44 +0000
@@ -35,6 +35,7 @@
 from bzrlib.transform import (TreeTransform, ROOT_PARENT, FinalPaths, 
                               resolve_conflicts, cook_conflicts, 
                               find_interesting, build_tree, get_backup_name)
+from bzrlib.workingtree import gen_root_id
 
 
 class TestTreeTransform(TestCaseInTempDir):
@@ -47,7 +48,7 @@
     def get_transform(self):
         transform = TreeTransform(self.wt)
         #self.addCleanup(transform.finalize)
-        return transform, transform.trans_id_tree_file_id(self.wt.get_root_id())
+        return transform, transform.root
 
     def test_existing_limbo(self):
         limbo_name = urlutils.local_path_from_url(
@@ -197,7 +198,7 @@
         transform3.delete_contents(oz_id)
         self.assertEqual(transform3.find_conflicts(), 
                          [('missing parent', oz_id)])
-        root_id = transform3.trans_id_tree_file_id('TREE_ROOT')
+        root_id = transform3.root
         tip_id = transform3.trans_id_tree_file_id('tip-id')
         transform3.adjust_path('tip', root_id, tip_id)
         transform3.apply()
@@ -229,7 +230,7 @@
     def test_name_invariants(self):
         create_tree, root = self.get_transform()
         # prepare tree
-        root = create_tree.trans_id_tree_file_id('TREE_ROOT')
+        root = create_tree.root
         create_tree.new_file('name1', root, 'hello1', 'name1')
         create_tree.new_file('name2', root, 'hello2', 'name2')
         ddir = create_tree.new_directory('dying_directory', root, 'ddir')
@@ -239,7 +240,7 @@
         create_tree.apply()
 
         mangle_tree,root = self.get_transform()
-        root = mangle_tree.trans_id_tree_file_id('TREE_ROOT')
+        root = mangle_tree.root
         #swap names
         name1 = mangle_tree.trans_id_tree_file_id('name1')
         name2 = mangle_tree.trans_id_tree_file_id('name2')
@@ -324,7 +325,7 @@
     def test_move_dangling_ie(self):
         create_tree, root = self.get_transform()
         # prepare tree
-        root = create_tree.trans_id_tree_file_id('TREE_ROOT')
+        root = create_tree.root
         create_tree.new_file('name1', root, 'hello1', 'name1')
         create_tree.apply()
         delete_contents, root = self.get_transform()
@@ -340,7 +341,7 @@
     def test_replace_dangling_ie(self):
         create_tree, root = self.get_transform()
         # prepare tree
-        root = create_tree.trans_id_tree_file_id('TREE_ROOT')
+        root = create_tree.root
         create_tree.new_file('name1', root, 'hello1', 'name1')
         create_tree.apply()
         delete_contents = TreeTransform(self.wt)
@@ -587,10 +588,11 @@
 
 
 class TransformGroup(object):
-    def __init__(self, dirname):
+    def __init__(self, dirname, root_id):
         self.name = dirname
         os.mkdir(dirname)
         self.wt = BzrDir.create_standalone_workingtree(dirname)
+        self.wt.set_root_id(root_id)
         self.b = self.wt.branch
         self.tt = TreeTransform(self.wt)
         self.root = self.tt.trans_id_tree_file_id(self.wt.get_root_id())
@@ -602,7 +604,8 @@
 
 class TestTransformMerge(TestCaseInTempDir):
     def test_text_merge(self):
-        base = TransformGroup("base")
+        root_id = gen_root_id()
+        base = TransformGroup("base", root_id)
         base.tt.new_file('a', base.root, 'a\nb\nc\nd\be\n', 'a')
         base.tt.new_file('b', base.root, 'b1', 'b')
         base.tt.new_file('c', base.root, 'c', 'c')
@@ -612,7 +615,7 @@
         base.tt.new_directory('g', base.root, 'g')
         base.tt.new_directory('h', base.root, 'h')
         base.tt.apply()
-        other = TransformGroup("other")
+        other = TransformGroup("other", root_id)
         other.tt.new_file('a', other.root, 'y\nb\nc\nd\be\n', 'a')
         other.tt.new_file('b', other.root, 'b2', 'b')
         other.tt.new_file('c', other.root, 'c2', 'c')
@@ -623,7 +626,7 @@
         other.tt.new_file('h', other.root, 'h\ni\nj\nk\n', 'h')
         other.tt.new_file('i', other.root, 'h\ni\nj\nk\n', 'i')
         other.tt.apply()
-        this = TransformGroup("this")
+        this = TransformGroup("this", root_id)
         this.tt.new_file('a', this.root, 'a\nb\nc\nd\bz\n', 'a')
         this.tt.new_file('b', this.root, 'b', 'b')
         this.tt.new_file('c', this.root, 'c', 'c')
@@ -680,9 +683,10 @@
     def test_file_merge(self):
         if not has_symlinks():
             raise TestSkipped('Symlinks are not supported on this platform')
-        base = TransformGroup("BASE")
-        this = TransformGroup("THIS")
-        other = TransformGroup("OTHER")
+        root_id = gen_root_id()
+        base = TransformGroup("BASE", root_id)
+        this = TransformGroup("THIS", root_id)
+        other = TransformGroup("OTHER", root_id)
         for tg in this, base, other:
             tg.tt.new_directory('a', tg.root, 'a')
             tg.tt.new_symlink('b', tg.root, 'b', 'b')
@@ -720,9 +724,10 @@
         self.assertIs(os.path.lexists(this.wt.abspath('h.OTHER')), True)
 
     def test_filename_merge(self):
-        base = TransformGroup("BASE")
-        this = TransformGroup("THIS")
-        other = TransformGroup("OTHER")
+        root_id = gen_root_id()
+        base = TransformGroup("BASE", root_id)
+        this = TransformGroup("THIS", root_id)
+        other = TransformGroup("OTHER", root_id)
         base_a, this_a, other_a = [t.tt.new_directory('a', t.root, 'a') 
                                    for t in [base, this, other]]
         base_b, this_b, other_b = [t.tt.new_directory('b', t.root, 'b') 
@@ -752,9 +757,10 @@
         self.assertEqual(this.wt.id2path('f'), pathjoin('b/f1'))
 
     def test_filename_merge_conflicts(self):
-        base = TransformGroup("BASE")
-        this = TransformGroup("THIS")
-        other = TransformGroup("OTHER")
+        root_id = gen_root_id()
+        base = TransformGroup("BASE", root_id)
+        this = TransformGroup("THIS", root_id)
+        other = TransformGroup("OTHER", root_id)
         base_a, this_a, other_a = [t.tt.new_directory('a', t.root, 'a') 
                                    for t in [base, this, other]]
         base_b, this_b, other_b = [t.tt.new_directory('b', t.root, 'b') 

=== modified file 'bzrlib/tests/test_tree.py'
--- bzrlib/tests/test_tree.py	2006-09-26 13:28:39 +0000
+++ bzrlib/tests/test_tree.py	2006-09-28 01:53:23 +0000
@@ -109,3 +109,12 @@
              ('compare', tree2, tree, 'unchanged', 'specific', 'extra',
               'require', True),
             ], RecordingOptimiser.calls)
+
+    def test_changes_from_with_root(self):
+        """Ensure the include_root option does what's expected."""
+        wt = self.make_branch_and_tree('.')
+        delta = wt.changes_from(wt.basis_tree())
+        self.assertEqual(len(delta.added), 0)
+        delta = wt.changes_from(wt.basis_tree(), wt, include_root=True)
+        self.assertEqual(len(delta.added), 1)
+        self.assertEqual(delta.added[0][0], '')

=== modified file 'bzrlib/tests/test_version_info.py'
--- bzrlib/tests/test_version_info.py	2006-09-22 05:38:28 +0000
+++ bzrlib/tests/test_version_info.py	2006-09-27 02:28:44 +0000
@@ -112,8 +112,8 @@
         self.assertEqual(['False'], stanza.get_all('clean'))
 
         file_rev_stanza = get_one_stanza(stanza, 'file-revisions')
-        self.assertEqual(['a', 'b', 'c'], file_rev_stanza.get_all('path'))
-        self.assertEqual(['r3', 'r2', 'unversioned'],
+        self.assertEqual(['', 'a', 'b', 'c'], file_rev_stanza.get_all('path'))
+        self.assertEqual(['r3', 'r3', 'r2', 'unversioned'],
             file_rev_stanza.get_all('revision'))
         os.remove('branch/c')
 
@@ -129,8 +129,10 @@
         wt.rename_one('b', 'd')
         stanza = regen(check_for_clean=True, include_file_revisions=True)
         file_rev_stanza = get_one_stanza(stanza, 'file-revisions')
-        self.assertEqual(['a', 'b', 'c', 'd'], file_rev_stanza.get_all('path'))
-        self.assertEqual(['modified', 'renamed to d', 'new', 'renamed from b'],
+        self.assertEqual(['', 'a', 'b', 'c', 'd'], 
+                          file_rev_stanza.get_all('path'))
+        self.assertEqual(['r3', 'modified', 'renamed to d', 'new', 
+                          'renamed from b'],
                          file_rev_stanza.get_all('revision'))
 
         wt.commit('modified', rev_id='r4')
@@ -138,8 +140,8 @@
         os.remove('branch/d')
         stanza = regen(check_for_clean=True, include_file_revisions=True)
         file_rev_stanza = get_one_stanza(stanza, 'file-revisions')
-        self.assertEqual(['a', 'c', 'd'], file_rev_stanza.get_all('path'))
-        self.assertEqual(['r4', 'unversioned', 'removed'],
+        self.assertEqual(['', 'a', 'c', 'd'], file_rev_stanza.get_all('path'))
+        self.assertEqual(['r4', 'r4', 'unversioned', 'removed'],
                          file_rev_stanza.get_all('revision'))
 
     def test_python_version(self):
@@ -182,7 +184,8 @@
         self.build_tree(['branch/c'])
         tvi = regen(check_for_clean=True, include_file_revisions=True)
         self.assertEqual(False, tvi.version_info['clean'])
-        self.assertEqual(['a', 'b', 'c'], sorted(tvi.file_revisions.keys()))
+        self.assertEqual(['', 'a', 'b', 'c'], 
+                         sorted(tvi.file_revisions.keys()))
         self.assertEqual('r3', tvi.file_revisions['a'])
         self.assertEqual('r2', tvi.file_revisions['b'])
         self.assertEqual('unversioned', tvi.file_revisions['c'])
@@ -199,7 +202,8 @@
         wt.add('c')
         wt.rename_one('b', 'd')
         tvi = regen(check_for_clean=True, include_file_revisions=True)
-        self.assertEqual(['a', 'b', 'c', 'd'], sorted(tvi.file_revisions.keys()))
+        self.assertEqual(['', 'a', 'b', 'c', 'd'], 
+                          sorted(tvi.file_revisions.keys()))
         self.assertEqual('modified', tvi.file_revisions['a'])
         self.assertEqual('renamed to d', tvi.file_revisions['b'])
         self.assertEqual('new', tvi.file_revisions['c'])
@@ -209,7 +213,8 @@
         wt.remove(['c', 'd'])
         os.remove('branch/d')
         tvi = regen(check_for_clean=True, include_file_revisions=True)
-        self.assertEqual(['a', 'c', 'd'], sorted(tvi.file_revisions.keys()))
+        self.assertEqual(['', 'a', 'c', 'd'], 
+                          sorted(tvi.file_revisions.keys()))
         self.assertEqual('r4', tvi.file_revisions['a'])
         self.assertEqual('unversioned', tvi.file_revisions['c'])
         self.assertEqual('removed', tvi.file_revisions['d'])

=== modified file 'bzrlib/tests/test_workingtree.py'
--- bzrlib/tests/test_workingtree.py	2006-09-15 02:03:15 +0000
+++ bzrlib/tests/test_workingtree.py	2006-09-22 04:52:17 +0000
@@ -173,9 +173,10 @@
         t = control.get_workingtree_transport(None)
         self.assertEqualDiff('Bazaar-NG Working Tree format 3',
                              t.get('format').read())
-        self.assertEqualDiff('<inventory format="5">\n'
-                             '</inventory>\n',
-                             t.get('inventory').read())
+        self.assertContainsRe(t.get('inventory').read(), 
+                              '<inventory file_id="[^"]*" format="5">\n'
+                              '</inventory>\n',
+                             )
         self.assertEqualDiff('### bzr hashcache v5\n',
                              t.get('stat-cache').read())
         self.assertFalse(t.has('inventory.basis'))

=== modified file 'bzrlib/tests/tree_implementations/__init__.py'
--- bzrlib/tests/tree_implementations/__init__.py	2006-09-05 11:52:44 +0000
+++ bzrlib/tests/tree_implementations/__init__.py	2006-09-10 19:04:48 +0000
@@ -86,13 +86,15 @@
         :param empty_tree: A working tree with no content and no parents to
             modify.
         """
+        empty_tree.set_root_id('empty-root-id')
         return self._convert_tree(empty_tree, converter)
 
     def _make_abc_tree(self, tree):
         """setup an abc content tree."""
         files = ['a', 'b/', 'b/c']
-        self.build_tree(files, line_endings='binary',
+        self.build_tree(files, line_endings='binary', 
                         transport=tree.bzrdir.root_transport)
+        tree.set_root_id('root-id')
         tree.add(files, ['a-id', 'b-id', 'c-id'])
 
     def get_tree_no_parents_abc_content(self, tree, converter=None):

=== modified file 'bzrlib/tests/tree_implementations/test_test_trees.py'
--- bzrlib/tests/tree_implementations/test_test_trees.py	2006-08-05 22:33:36 +0000
+++ bzrlib/tests/tree_implementations/test_test_trees.py	2006-08-06 00:20:32 +0000
@@ -16,7 +16,6 @@
 
 """Tests for the test trees used by the tree_implementations tests."""
 
-from bzrlib import inventory 
 from bzrlib.tests.tree_implementations import TestCaseWithTree
 
 
@@ -28,9 +27,9 @@
         self.assertEqual([], tree.get_parent_ids())
         self.assertEqual([], tree.conflicts())
         self.assertEqual([], list(tree.unknowns()))
-        self.assertEqual([inventory.ROOT_ID], list(iter(tree)))
+        self.assertEqual(['empty-root-id'], list(iter(tree)))
         self.assertEqual(
-            [('', inventory.ROOT_ID)],
+            [('', 'empty-root-id')],
             [(path, node.file_id) for path, node in tree.iter_entries_by_dir()])
 
     def test_abc_tree_no_parents(self):
@@ -41,10 +40,10 @@
         self.assertEqual([], list(tree.unknowns()))
         # __iter__ has no strongly defined order
         self.assertEqual(
-            set([inventory.ROOT_ID, 'a-id', 'b-id', 'c-id']),
+            set(['root-id', 'a-id', 'b-id', 'c-id']),
             set(iter(tree)))
         self.assertEqual(
-            [('', inventory.ROOT_ID), ('a', 'a-id'), ('b', 'b-id'), ('b/c', 'c-id')],
+            [('', 'root-id'), ('a', 'a-id'), ('b', 'b-id'), ('b/c', 'c-id')],
             [(path, node.file_id) for path, node in tree.iter_entries_by_dir()])
         self.assertEqualDiff('contents of a\n', tree.get_file_text('a-id'))
         self.assertFalse(tree.is_executable('c-id'))
@@ -57,10 +56,10 @@
         self.assertEqual([], list(tree.unknowns()))
         # __iter__ has no strongly defined order
         self.assertEqual(
-            set([inventory.ROOT_ID, 'a-id', 'b-id', 'c-id']),
+            set(['root-id', 'a-id', 'b-id', 'c-id']),
             set(iter(tree)))
         self.assertEqual(
-            [('', inventory.ROOT_ID), ('a', 'a-id'), ('b', 'b-id'), ('b/c', 'c-id')],
+            [('', 'root-id'), ('a', 'a-id'), ('b', 'b-id'), ('b/c', 'c-id')],
             [(path, node.file_id) for path, node in tree.iter_entries_by_dir()])
         self.assertEqualDiff('foobar\n', tree.get_file_text('a-id'))
         self.assertFalse(tree.is_executable('c-id'))
@@ -73,10 +72,10 @@
         self.assertEqual([], list(tree.unknowns()))
         # __iter__ has no strongly defined order
         self.assertEqual(
-            set([inventory.ROOT_ID, 'a-id', 'b-id', 'c-id']),
+            set(['root-id', 'a-id', 'b-id', 'c-id']),
             set(iter(tree)))
         self.assertEqual(
-            [('', inventory.ROOT_ID), ('a', 'a-id'), ('b', 'b-id'), ('b/c', 'c-id')],
+            [('', 'root-id'), ('a', 'a-id'), ('b', 'b-id'), ('b/c', 'c-id')],
             [(path, node.file_id) for path, node in tree.iter_entries_by_dir()])
         self.assertEqualDiff('contents of a\n', tree.get_file_text('a-id'))
         self.assertTrue(tree.is_executable('c-id'))
@@ -89,10 +88,10 @@
         self.assertEqual([], list(tree.unknowns()))
         # __iter__ has no strongly defined order
         self.assertEqual(
-            set([inventory.ROOT_ID, 'a-id', 'b-id', 'c-id']),
+            set(['root-id', 'a-id', 'b-id', 'c-id']),
             set(iter(tree)))
         self.assertEqual(
-            [('', inventory.ROOT_ID), ('b', 'b-id'), ('d', 'a-id'), ('b/c', 'c-id')],
+            [('', 'root-id'), ('b', 'b-id'), ('d', 'a-id'), ('b/c', 'c-id')],
             [(path, node.file_id) for path, node in tree.iter_entries_by_dir()])
         self.assertEqualDiff('contents of a\n', tree.get_file_text('a-id'))
         self.assertFalse(tree.is_executable('c-id'))
@@ -105,10 +104,10 @@
         self.assertEqual([], list(tree.unknowns()))
         # __iter__ has no strongly defined order
         self.assertEqual(
-            set([inventory.ROOT_ID, 'a-id', 'b-id', 'c-id']),
+            set(['root-id', 'a-id', 'b-id', 'c-id']),
             set(iter(tree)))
         self.assertEqual(
-            [('', inventory.ROOT_ID), ('b', 'b-id'), ('d', 'a-id'), ('b/c', 'c-id')],
+            [('', 'root-id'), ('b', 'b-id'), ('d', 'a-id'), ('b/c', 'c-id')],
             [(path, node.file_id) for path, node in tree.iter_entries_by_dir()])
         self.assertEqualDiff('bar\n', tree.get_file_text('a-id'))
         self.assertFalse(tree.is_executable('c-id'))
@@ -121,10 +120,10 @@
         self.assertEqual([], list(tree.unknowns()))
         # __iter__ has no strongly defined order
         self.assertEqual(
-            set([inventory.ROOT_ID, 'a-id', 'b-id', 'c-id']),
+            set(['root-id', 'a-id', 'b-id', 'c-id']),
             set(iter(tree)))
         self.assertEqual(
-            [('', inventory.ROOT_ID), ('a', 'a-id'), ('b', 'b-id'), ('e', 'c-id')],
+            [('', 'root-id'), ('a', 'a-id'), ('b', 'b-id'), ('e', 'c-id')],
             [(path, node.file_id) for path, node in tree.iter_entries_by_dir()])
         self.assertEqualDiff('contents of a\n', tree.get_file_text('a-id'))
         self.assertTrue(tree.is_executable('c-id'))

=== modified file 'bzrlib/tests/workingtree_implementations/test_workingtree.py'
--- bzrlib/tests/workingtree_implementations/test_workingtree.py	2006-09-08 00:19:48 +0000
+++ bzrlib/tests/workingtree_implementations/test_workingtree.py	2006-09-10 19:04:48 +0000
@@ -413,6 +413,28 @@
         self.failUnlessExists('checkout/file')
         self.assertEqual(['A'], old_tree.get_parent_ids())
 
+    def test_update_sets_root_id(self):
+        """Ensure tree root is set properly by update.
+        
+        Since empty trees don't have root_ids, but workingtrees do,
+        an update of a checkout of revision 0 to a new revision,  should set
+        the root id.
+        """
+        wt = self.make_branch_and_tree('tree')
+        main_branch = wt.branch
+        # create an out of date working tree by making a checkout in this
+        # current format
+        self.build_tree(['checkout/', 'tree/file'])
+        checkout = main_branch.create_checkout('checkout')
+        # now commit to 'tree'
+        wt.add('file')
+        wt.commit('A', rev_id='A')
+        # and update checkout 
+        self.assertEqual(0, checkout.update())
+        self.failUnlessExists('checkout/file')
+        self.assertEqual(wt.get_root_id(), checkout.get_root_id())
+        self.assertNotEqual(None, wt.get_root_id())
+
     def test_update_returns_conflict_count(self):
         # working tree formats from the meta-dir format and newer support
         # setting the last revision on a tree independently of that on the 

=== modified file 'bzrlib/textinv.py'
--- bzrlib/textinv.py	2006-08-07 23:14:45 +0000
+++ bzrlib/textinv.py	2006-08-07 23:19:38 +0000
@@ -52,9 +52,10 @@
 def write_text_inventory(inv, outf):
     """Write out inv in a simple trad-unix text format."""
     outf.write(START_MARK)
-    entries = inv.iter_entries()
-    entries.next()
-    for path, ie in entries:
+    for path, ie in inv.iter_entries():
+        if inv.is_root(ie.file_id):
+            continue
+        
         outf.write(ie.file_id + ' ')
         outf.write(escape(ie.name) + ' ')
         outf.write(ie.kind + ' ')

=== modified file 'bzrlib/transform.py'
--- bzrlib/transform.py	2006-09-27 19:14:52 +0000
+++ bzrlib/transform.py	2006-09-28 01:53:23 +0000
@@ -21,7 +21,7 @@
 from bzrlib import bzrdir, errors
 from bzrlib.errors import (DuplicateKey, MalformedTransform, NoSuchFile,
                            ReusingTransform, NotVersionedError, CantMoveRoot,
-                           ExistingLimbo, ImmortalLimbo)
+                           ExistingLimbo, ImmortalLimbo, NoFinalPath)
 from bzrlib.inventory import InventoryEntry
 from bzrlib.osutils import (file_kind, supports_executable, pathjoin, lexists,
                             delete_any)
@@ -457,7 +457,10 @@
         try:
             return self._new_name[trans_id]
         except KeyError:
-            return os.path.basename(self._tree_id_paths[trans_id])
+            try:
+                return os.path.basename(self._tree_id_paths[trans_id])
+            except KeyError:
+                raise NoFinalPath(trans_id, self)
 
     def by_parent(self):
         """Return a map of parent: children for known parents.
@@ -570,7 +573,10 @@
             parent_id = trans_id
             while parent_id is not ROOT_PARENT:
                 seen.add(parent_id)
-                parent_id = self.final_parent(parent_id)
+                try:
+                    parent_id = self.final_parent(parent_id)
+                except KeyError:
+                    break
                 if parent_id == trans_id:
                     conflicts.append(('parent loop', trans_id))
                 if parent_id in seen:
@@ -958,6 +964,8 @@
     file_trans_id = {}
     top_pb = bzrlib.ui.ui_factory.nested_progress_bar()
     pp = ProgressPhase("Build phase", 2, top_pb)
+    if tree.inventory.root is not None:
+        wt.set_root_id(tree.inventory.root.file_id)
     tt = TreeTransform(wt)
     divert = set()
     try:
@@ -1232,6 +1240,9 @@
         basis_tree = None
         try:
             for id_num, file_id in enumerate(wt_interesting):
+                if (working_tree.inventory.is_root(file_id) and 
+                    len(target_tree.inventory) == 0):
+                    continue
                 child_pb.update("New file check", id_num+1, 
                                 len(sorted_interesting))
                 if file_id not in target_tree:

=== modified file 'bzrlib/tree.py'
--- bzrlib/tree.py	2006-09-21 22:02:58 +0000
+++ bzrlib/tree.py	2006-09-28 22:56:04 +0000
@@ -218,7 +218,7 @@
 class EmptyTree(Tree):
 
     def __init__(self):
-        self._inventory = Inventory()
+        self._inventory = Inventory(root_id=None)
         warn('EmptyTree is deprecated as of bzr 0.9 please use '
             'repository.revision_tree instead.',
             DeprecationWarning, stacklevel=2)
@@ -236,7 +236,7 @@
         assert self._inventory[file_id].kind == "directory"
         return "directory"
 
-    def list_files(self):
+    def list_files(self, include_root=False):
         return iter([])
     
     def __contains__(self, file_id):

=== modified file 'bzrlib/treebuilder.py'
--- bzrlib/treebuilder.py	2006-09-15 02:03:15 +0000
+++ bzrlib/treebuilder.py	2006-09-22 04:52:17 +0000
@@ -31,6 +31,7 @@
     def __init__(self):
         """Construct a TreeBuilder."""
         self._tree = None
+        self._root_done = False
 
     def build(self, recipe):
         """Build recipe into the current tree.
@@ -40,6 +41,9 @@
             '/' then a directory is added, otherwise a regular file is added.
         """
         self._ensure_building()
+        if not self._root_done:
+            self._tree.add('', 'root-id', 'directory')
+            self._root_done = True
         for name in recipe:
             assert isinstance(name, basestring)
             if name[-1] == '/':
@@ -73,4 +77,3 @@
             raise errors.AlreadyBuilding
         self._tree = tree
         self._tree.lock_tree_write()
-

=== modified file 'bzrlib/version_info_formats/__init__.py'
--- bzrlib/version_info_formats/__init__.py	2006-09-21 16:33:39 +0000
+++ bzrlib/version_info_formats/__init__.py	2006-09-29 00:04:13 +0000
@@ -85,13 +85,14 @@
             basis_tree = self._branch.basis_tree()
 
         # Build up the list from the basis inventory
-        for info in basis_tree.list_files():
+        for info in basis_tree.list_files(include_root=True):
             self._file_revisions[info[0]] = info[-1].revision
 
         if not self._check or self._working_tree is None:
             return
 
-        delta = self._working_tree.changes_from(basis_tree)
+        delta = self._working_tree.changes_from(basis_tree, 
+                                                include_root=True)
 
         # Using a 2-pass algorithm for renames. This is because you might have
         # renamed something out of the way, and then created a new file

=== modified file 'bzrlib/workingtree.py'
--- bzrlib/workingtree.py	2006-10-06 06:27:06 +0000
+++ bzrlib/workingtree.py	2006-10-15 15:59:57 +0000
@@ -164,7 +164,7 @@
 
 def gen_root_id():
     """Return a new tree-root file id."""
-    return gen_file_id('TREE_ROOT')
+    return gen_file_id('tree_root')
 
 
 class TreeEntry(object):
@@ -538,6 +538,7 @@
     @needs_read_lock
     def copy_content_into(self, tree, revision_id=None):
         """Copy the current content and user files of this tree into tree."""
+        tree.set_root_id(self.get_root_id())
         if revision_id is None:
             merge.transform_tree(tree, self)
         else:
@@ -1158,6 +1159,9 @@
                                 basis_tree,
                                 this_tree=self,
                                 pb=pb)
+                    if (basis_tree.inventory.root is None and
+                        new_basis_tree.inventory.root is not None):
+                        self.set_root_id(new_basis_tree.inventory.root.file_id)
                 finally:
                     pb.finished()
                 # TODO - dedup parents list with things merged by pull ?
@@ -1588,23 +1592,30 @@
     def update(self):
         """Update a working tree along its branch.
 
-        This will update the branch if its bound too, which means we have multiple trees involved:
-        The new basis tree of the master.
-        The old basis tree of the branch.
-        The old basis tree of the working tree.
-        The current working tree state.
-        pathologically all three may be different, and non ancestors of each other.
-        Conceptually we want to:
-        Preserve the wt.basis->wt.state changes
-        Transform the wt.basis to the new master basis.
-        Apply a merge of the old branch basis to get any 'local' changes from it into the tree.
-        Restore the wt.basis->wt.state changes.
+        This will update the branch if its bound too, which means we have
+        multiple trees involved:
+
+        - The new basis tree of the master.
+        - The old basis tree of the branch.
+        - The old basis tree of the working tree.
+        - The current working tree state.
+
+        Pathologically, all three may be different, and non-ancestors of each
+        other.  Conceptually we want to:
+
+        - Preserve the wt.basis->wt.state changes
+        - Transform the wt.basis to the new master basis.
+        - Apply a merge of the old branch basis to get any 'local' changes from
+          it into the tree.
+        - Restore the wt.basis->wt.state changes.
 
         There isn't a single operation at the moment to do that, so we:
-        Merge current state -> basis tree of the master w.r.t. the old tree basis.
-        Do a 'normal' merge of the old branch basis if it is relevant.
+        - Merge current state -> basis tree of the master w.r.t. the old tree
+          basis.
+        - Do a 'normal' merge of the old branch basis if it is relevant.
         """
         old_tip = self.branch.update()
+
         # here if old_tip is not None, it is the old tip of the branch before
         # it was updated from the master branch. This should become a pending
         # merge in the working tree to preserve the user existing work.  we
@@ -1624,6 +1635,8 @@
             # merge tree state up to new branch tip.
             basis = self.basis_tree()
             to_tree = self.branch.basis_tree()
+            if basis.inventory.root is None:
+                self.set_root_id(to_tree.inventory.root.file_id)
             result += merge.merge_inner(
                                   self.branch,
                                   to_tree,
@@ -1957,9 +1970,11 @@
                          _internal=True,
                          _format=self,
                          _bzrdir=a_bzrdir)
+        wt.set_last_revision(revision)
+        basis_tree = wt.basis_tree()
+        if basis_tree.inventory.root is not None:
+            inv.root.file_id = basis_tree.inventory.root.file_id
         wt._write_inventory(inv)
-        wt.set_root_id(inv.root.file_id)
-        basis_tree = branch.repository.revision_tree(revision)
         wt.set_parent_trees([(revision, basis_tree)])
         transform.build_tree(basis_tree, wt)
         return wt
@@ -2029,7 +2044,7 @@
         branch = a_bzrdir.open_branch()
         if revision_id is None:
             revision_id = branch.last_revision()
-        inv = Inventory() 
+        inv = Inventory(root_id=gen_root_id()) 
         wt = WorkingTree3(a_bzrdir.root_transport.local_abspath('.'),
                          branch,
                          inv,
@@ -2039,10 +2054,11 @@
                          _control_files=control_files)
         wt.lock_tree_write()
         try:
+            wt.set_last_revision(revision_id)
+            basis_tree = wt.basis_tree()
             wt._write_inventory(inv)
-            wt.set_root_id(inv.root.file_id)
-            basis_tree = branch.repository.revision_tree(revision_id)
-            if revision_id == NULL_REVISION:
+            wt.set_pending_merges([])
+            if revision_id == bzrlib.revision.NULL_REVISION:
                 wt.set_parent_trees([])
             else:
                 wt.set_parent_trees([(revision_id, basis_tree)])

